<?php
/**
 * @author      Lefteris Kavadas
 * @copyright   Copyright (c) 2016 - 2018 Lefteris Kavadas / firecoders.com
 * @license     GNU General Public License version 3 or later
 */
defined('_JEXEC') or die;

class plgSystemRoute66PageSpeed extends JPlugin
{
    protected $script = '';
    protected $style = '';
    protected $optimize = false;
    protected $optimizeJs = false;
    protected $optimizeCss = false;
    protected $cache;
    protected $cacheId;

    public function onAfterInitialise()
    {
        $application = JFactory::getApplication();
        if ($application->isSite())
        {
            $componentParams = JComponentHelper::getParams('com_route66');
            $this->params->merge($componentParams);
        }
    }

    public function onAfterDispatch()
    {
        $application = JFactory::getApplication();
        if($application->isSite())
        {
          $document = JFactory::getDocument();
          $user = JFactory::getUser();
          $option = $application->input->getCmd('option');
          $this->optimize = $user->guest && $application->input->getMethod() == 'GET' && $document->getType() == 'html' && !in_array($option, $this->params->get('optimizeExclusions', array()));
          $this->optimizeJs = $this->optimize && $this->params->get('optimizeJs');
          $this->optimizeCss = $this->optimize && $this->params->get('optimizeCss');
          $this->cacheId = md5(JUri::current());
          $this->cache = JFactory::getCache('plg_system_route66_pagespeed', 'output');
          $this->cache->setCaching(true);
          $this->cache->setLifeTime($this->params->get('optimizeCacheTime', 15));
        }

    }

    public function onBeforeCompileHead()
    {
        $document = JFactory::getDocument();
        $rootRelative = JUri::root(true);
        $rootAbsolute = JUri::root(false);

        if ($this->optimizeJs)
        {
            foreach ($document->_scripts as $key => $script)
            {
                $script['defer'] = true;
                $document->_scripts[$key] = $script;
            }

            foreach ($document->_script as $type => $content)
            {
                if ($type == 'text/javascript' || $type == '')
                {
                    $this->script .= $content;
                    unset($document->_script[$type]);
                }
            }
        }

        if ($this->optimizeCss)
        {
            $this->style = $this->cache->get($this->cacheId.'_style');

            if (!$this->style)
            {
                $this->style = '';
                foreach ($document->_styleSheets as $key => $stylesheet)
                {
                    $href = $key;

                    if (strpos($href, '//') === 0)
                    {
                        continue;
                    }

                    if (strpos($href, 'http') === 0 && strpos($href, $rootAbsolute) !== 0)
                    {
                        continue;
                    }

                    if (strpos($href, $rootAbsolute) === 0)
                    {
                        $href = str_replace($rootAbsolute, '', $href);
                    }
                    elseif ($rootRelative && strpos($href, $rootRelative) === 0)
                    {
                        $href = str_replace($rootRelative.'/', '', $href);
                    }

                    $filepath = parse_url($href, PHP_URL_PATH);
                    $basepath = dirname($filepath);
                    $parts = explode('/', $basepath);
                    $css = @file_get_contents(JPATH_SITE.'/'.$filepath);

                    preg_match_all('/url\((.*?)\)/i', $css, $matches);
                    if (isset($matches[1]))
                    {
                        $replacements = array();
                        foreach ($matches[1] as $url)
                        {
                            $url = trim($url, '\'"');

                            if (strpos($url, 'data:') === 0)
                            {
                                $replacements[] = $url;
                                continue;
                            }

                            $count = substr_count($url, '../');
                            if ($count)
                            {
                                $url = str_replace(str_repeat('../', $count), implode('/', array_slice($parts, 0, -$count)).'/', $url);
                                $url = ltrim($url, '/');
                            }
                            elseif (strpos($url, './') === 0)
                            {
                                $url = str_replace('./', $basepath.'/', $url);
                            }
                            elseif (strpos($url, '/') === 0)
                            {
                                $url = $url;
                            }
                            else
                            {
                                $url = $basepath.'/'.$url;
                            }

                            $replacements[] = $rootRelative.'/'.$url;
                        }

                        $this->style .= str_replace($matches[1], $replacements, $css);
                    }

                    unset($document->_styleSheets[$key]);
                }

                foreach ($document->_style as $key => $css)
                {
                    $this->style .= $css;
                    unset($document->_style[$key]);
                }

                if ($this->style && $this->params->get('minifyCss'))
                {
                    require_once JPATH_SITE.'/plugins/system/route66pagespeed/lib/minifier.php';
                    $minifier = new Route66CssMinifier();
                    $this->style = $minifier->process($this->style);
                }

                $this->cache->store($this->style, $this->cacheId.'_style');

            }
            else
            {
                foreach ($document->_styleSheets as $key => $entry)
                {
                    if (strpos($key, '//') === 0)
                    {
                        continue;
                    }

                    if (strpos($key, 'http') === 0 && strpos($key, $rootAbsolute) !== 0)
                    {
                        continue;
                    }

                    unset($document->_styleSheets[$key]);
                }

                foreach ($document->_style as $key => $entry)
                {
                    unset($document->_style[$key]);
                }
            }
        }
    }


    public function onAfterRender()
    {
        $application = JFactory::getApplication();
        $buffer = $application->getBody();
        $searches = array();
        $replacements = array();

        if ($this->optimizeJs)
        {
            $scripts = preg_match_all("/<script((?:(?!src=).)*?)>(.*?)<\/script>/smix", $buffer, $matches);

            foreach ($matches[0] as $key => $script)
            {
                $attributes = $matches[1][$key];
                if (strpos($attributes, 'type') === false || strpos($attributes, 'text/javascript'))
                {
                    $content = $matches[2][$key];
                    $this->script .= $content;
                    $searches[] = $script;
                    $replacements[] = '';
                }
            }

            if ($this->script)
            {
                if ($this->params->get('optimizeJsMethod') == 'file')
                {
                    $timestamp = $this->cache->get($this->cacheId.'_script_timestamp');
                    if (!$timestamp)
                    {
                        $date = JFactory::getDate();
                        $timestamp = $date->toUnix();
                        $this->cache->store($timestamp, $this->cacheId.'_script_timestamp');
                        jimport('joomla.filesystem.file');
                        JFile::write(JPATH_SITE.'/media/route66/scripts/'.$this->cacheId.'.js', $this->script);
                    }
                    $script = '<script src="'.JUri::root(true).'/media/route66/scripts/'.$this->cacheId.'.js?v='.$timestamp.'" defer></script>';
                }
                else
                {
                    $script = '<script>document.addEventListener("DOMContentLoaded", function(event) {'.$this->script.'});</script>';
                }
                $searches[] = '</body>';
                $replacements[] = $script.'</body>';
            }
        }

        if ($this->optimizeCss && $this->style)
        {
            $searches[] = '</head>';
            $replacements[] = '<style>'.$this->style.'</style></head>';
        }

        if (count($searches))
        {
            $buffer = str_replace($searches, $replacements, $buffer);
            $application->setBody($buffer);
        }

        return true;
    }
}
